#synthDrivers/ttusbd.py
#A part of NonVisual Desktop Access (NVDA)
#copyright 2012 by NVDA contributers etc
#www.nvda-project.org/
#TripleTalk USB Driver
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#Todo: Make indexing functional; right now it's just sort of existing in code, but not in true form :D


from cStringIO import StringIO
import threading
import synthDriverHandler
from synthDriverHandler import SynthDriver,VoiceInfo
from collections import OrderedDict
from logHandler import log
from ctypes import *
from ctypes.wintypes import *
import config
import speech
from winUser import WNDCLASSEXW, WNDPROC
#import tones
import logging

USBTT = cdll.ttusbd
WM_INDEX = 0x401
lastIndex = None


class IndexingThread(threading.Thread):
	def run(self):
		global lastindex
		while True:
			lastindex = USBTT.USBTT_ReadByte()


class SynthDriver(synthDriverHandler.SynthDriver):
	name="ttusb"
	description=_("TripleTalk USB")
	supportedSettings=(SynthDriver.RateSetting(10),SynthDriver.PitchSetting(10),SynthDriver.InflectionSetting(10))
	minRate = 0
	maxRate = 9
	minPitch=0
	maxPitch=99
	minInflection=0
	maxInflection=9

	@classmethod
	def check(cls):
		USBTT.USBTT_WriteByte(24)
		return True

	def __init__(self):
		super(synthDriverHandler.SynthDriver,self).__init__()
		self.indexingThread = IndexingThread()
		self.indexingThread.start()
		self.tt_rate = 4
		self.tt_pitch=50
		self.tt_inflection=5

	def speak(self, speechSequence):
		text_list = []
		for item in speechSequence:
			if isinstance(item, basestring):
				text_list.append(item)
			elif isinstance(item,speech.IndexCommand):
				text_list.append("\x01%di" % item.index)

		#No unicode here. Do something better with this later
		text = u" ".join(text_list).encode('ascii', 'replace')
		text = "\x01%ss \x01%sp \x01%se %s" % (self.tt_rate, self.tt_pitch, self.tt_inflection, text)
		if text:
#clean this up later, special params I use for better sounding voice
			USBTT.USBTT_WriteString('\x012a\x012x\x018v'+text+'\r', len('\x010a\x012x'+text+'\rlll'))

	def cancel(self):
		USBTT.USBTT_WriteString('\x25',1)
		USBTT.USBTT_WriteString('\x18',1)
		USBTT.USBTT_WriteByteImmediate()

	def _set_rate(self, rate):
		val = self._percentToParam(rate, self.minRate, self.maxRate)
		self.tt_rate = val

	def _get_rate(self):
		return self._paramToPercent(self.tt_rate, self.minRate, self.maxRate)

	def _set_pitch(self, pitch):
		val = self._percentToParam(pitch, self.minPitch, self.maxPitch)
		self.tt_pitch = val

	def _get_pitch(self):
		return self._paramToPercent(self.tt_pitch, self.minPitch, self.maxPitch)

	def _set_inflection(self, inflection):
		val = self._percentToParam(inflection, self.minInflection, self.maxInflection)
		self.tt_inflection = val

	def _get_inflection(self):
		return self._paramToPercent(self.tt_inflection, self.minInflection, self.maxInflection)

	def _get_lastIndex(self):
#		print(USBTT.USBTT_ReadByte())
		return USBTT.USBTT_ReadByte()

